@@ -1,6 +1,6 @@
 /* Python interface to objfiles.
 
-   Copyright (C) 2008-2024 Free Software Foundation, Inc.
+   Copyright (C) 2008-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -26,17 +26,11 @@
 #include "python.h"
 #include "inferior.h"
 
-struct objfile_object
+struct objfile_object : public gdbpy_dict_wrapper
 {
-  PyObject_HEAD
-
   /* The corresponding objfile.  */
   struct objfile *objfile;
 
-  /* Dictionary holding user-added attributes.
-     This is the __dict__ attribute of the object.  */
-  PyObject *dict;
-
   /* The pretty-printer list of functions.  */
   PyObject *printers;
 
@@ -53,8 +47,7 @@ struct objfile_object
   PyObject *xmethods;
 };
 
-extern PyTypeObject objfile_object_type
-    CPYCHECKER_TYPE_OBJECT_FOR_TYPEDEF ("objfile_object");
+extern PyTypeObject objfile_object_type;
 
 /* Clear the OBJFILE pointer in an Objfile object and remove the
    reference.  */
@@ -556,7 +549,7 @@ objfpy_build_id_ok (const char *string)
     return 0;
   for (i = 0; i < n; ++i)
     {
-      if (!isxdigit (string[i]))
+      if (!c_isxdigit (string[i]))
 	return 0;
     }
   return 1;
@@ -619,9 +612,8 @@ gdbpy_lookup_objfile (PyObject *self, PyObject *args, PyObject *kw)
 
   struct objfile *objfile = nullptr;
   if (by_build_id)
-    gdbarch_iterate_over_objfiles_in_search_order
-      (current_inferior ()->arch (),
-       [&objfile, name] (struct objfile *obj)
+    current_program_space->iterate_over_objfiles_in_search_order
+      ([&objfile, name] (struct objfile *obj)
 	 {
 	   /* Don't return separate debug files.  */
 	   if (obj->separate_debug_objfile_backlink != nullptr)
@@ -642,9 +634,8 @@ gdbpy_lookup_objfile (PyObject *self, PyObject *args, PyObject *kw)
 	   return 1;
 	 }, gdbpy_current_objfile);
   else
-    gdbarch_iterate_over_objfiles_in_search_order
-      (current_inferior ()->arch (),
-       [&objfile, name] (struct objfile *obj)
+    current_program_space->iterate_over_objfiles_in_search_order
+      ([&objfile, name] (struct objfile *obj)
 	 {
 	   /* Don't return separate debug files.  */
 	   if (obj->separate_debug_objfile_backlink != nullptr)
@@ -706,8 +697,8 @@ objfile_to_objfile_object (struct objfile *objfile)
   return gdbpy_ref<>::new_reference (result);
 }
 
-static int CPYCHECKER_NEGATIVE_RESULT_SETS_EXCEPTION
-gdbpy_initialize_objfile (void)
+static int
+gdbpy_initialize_objfile ()
 {
   return gdbpy_type_ready (&objfile_object_type);
 }
@@ -742,8 +733,8 @@ Look up a static-linkage global symbol in this objfile and return it." },
 
 static gdb_PyGetSetDef objfile_getset[] =
 {
-  { "__dict__", gdb_py_generic_dict, NULL,
-    "The __dict__ for this objfile.", &objfile_object_type },
+  { "__dict__", gdb_py_generic_dict_getter, NULL,
+    "The __dict__ for this objfile.", NULL },
   { "filename", objfpy_get_filename, NULL,
     "The objfile's filename, or None.", NULL },
   { "username", objfpy_get_username, NULL,
@@ -788,8 +779,8 @@ PyTypeObject objfile_object_type =
   0,				  /*tp_hash */
   0,				  /*tp_call*/
   0,				  /*tp_str*/
-  0,				  /*tp_getattro*/
-  0,				  /*tp_setattro*/
+  gdb_py_generic_getattro,	  /*tp_getattro*/
+  gdb_py_generic_setattro,	  /*tp_setattro*/
   0,				  /*tp_as_buffer*/
   Py_TPFLAGS_DEFAULT,		  /*tp_flags*/
   "GDB objfile object",		  /* tp_doc */
@@ -806,7 +797,7 @@ PyTypeObject objfile_object_type =
   0,				  /* tp_dict */
   0,				  /* tp_descr_get */
   0,				  /* tp_descr_set */
-  offsetof (objfile_object, dict), /* tp_dictoffset */
+  0,				  /* tp_dictoffset */
   0,				  /* tp_init */
   0,				  /* tp_alloc */
   objfpy_new,			  /* tp_new */
